package MyApp::YouTube;
use strict;
use warnings;
use base qw/Class::Accessor::Fast Class::ErrorHandler/;

our $VERSION = 0.01;

use HTTP::Response::Encoding;
use LWP::UserAgent;
use Params::Validate qw/:all/;
use URI;
use XML::Atom::Feed;
use XML::Atom::Util qw/nodelist/;

use MyApp::YouTube::Result;

__PACKAGE__->mk_accessors(qw/title link thumbnail_url/);

my $media = XML::Atom::Namespace->new(media => 'http://search.yahoo.com/mrss/');

sub search {
    my $class = shift;
    my ($query) = validate_pos(@_, { type => SCALAR });

    my $uri = URI->new('http://gdata.youtube.com/feeds/api/videos');
    $uri->query_form(vq => $query, orderby => 'viewCount' );

    my $ua = LWP::UserAgent->new;
    $ua->agent( join '/', __PACKAGE__, __PACKAGE__->VERSION );

    my $res = $ua->get($uri);
    if ($res->is_error) {
        return $res->status_line;
    }

    my @entries = $class->_parse_videos( \$res->decoded_content );
    return MyApp::YouTube::Result->new(\@entries);
}

sub _parse_videos {
    my $class = shift;
    my ($ref_xml) = validate_pos(@_, { type => SCALARREF });
    my $feed = XML::Atom::Feed->new($ref_xml) or die XML::Atom->errstr;

    my @entries;
    for my $entry ($feed->entries) {
        push @entries, $class->new({
            title         => $entry->title,
            link          => $entry->link->href,
            thumbnail_url => thumbnail($entry),
        });
    }
    return @entries;
}

sub thumbnail {
    my ($entry) = validate_pos(@_, { isa => 'XML::Atom::Entry' });

    my @nodes = nodelist($entry->elem, $media->{uri}, 'thumbnail');
    return $nodes[0]->getAttribute('url');
}

sub to_hash {
    my $self = shift;
    return {
        title         => $self->title,
        thumbnail_url => $self->thumbnail_url,
        link          => $self->link
    }
}

1;
