#!/usr/local/bin/perl
use strict;
use warnings;
use encoding 'utf-8';

use CGI qw/:standard/;
use HTTP::Response::Encoding;
use LWP::UserAgent;
use Params::Validate qw/:all/;
use URI;
use XML::Atom::Feed;
use XML::Atom::Util qw/nodelist/;
use JSON::Syck;

my $q = CGI->new;

if ($q->param('json') and $q->param('q')) {
    print
        $q->header( -type => 'application/json', -charset => 'utf-8' ),
        json( search_video($q->param('q'))->entries );
} else {
my $style =<<EOF;
ul li {
  list-style-type: none;
  display: inline;
}
EOF

    print
        $q->header ( -type => 'text/html', -charset => 'utf-8' ),
        start_html ( -title => 'YouTube Search', -style => { -code => $style } ),
        h1         ( 'YouTube Search' ),
        start_form ( -method => 'get' ),
        textfield  ( -name => 'q', -value => $q->param('q') || '' ),
        submit     ( -value => '検索' ),
        end_form;

    if (my $query = $q->param('q')) {
        print
            h2('検索結果'),
            ul( li [ map anchor($_), search_video( $query )->entries ] );
    }
    print end_html;
}

sub anchor {
    my ($entry) = validate_pos(@_, { isa => 'XML::Atom::Entry'});
    return a( {href => $entry->link->href}, img { src => thumbnail($entry), title => $entry->title } )
}

sub search_video {
    my ($query) = validate_pos(@_, { type => SCALAR });
    my $uri = URI->new('http://gdata.youtube.com/feeds/api/videos');
    $uri->query_form(vq => $query, orderby => 'viewCount' );

    my $ua = LWP::UserAgent->new;
    $ua->agent('sample application/0.01');

    my $res = $ua->get($uri);
    if ($res->is_error) {
        die $res->status_line;
    }

    return _parse_videos( \$res->decoded_content );
}

sub _parse_videos {
    my ($ref_xml) = validate_pos(@_, { type => SCALARREF });
    return XML::Atom::Feed->new($ref_xml) or die XML::Atom->errstr;
}

my $media = XML::Atom::Namespace->new(media => 'http://search.yahoo.com/mrss/');

sub thumbnail {
    my ($entry) = validate_pos(@_, { isa => 'XML::Atom::Entry' });

    my @nodes = nodelist($entry->elem, $media->{uri}, 'thumbnail');
    return $nodes[0]->getAttribute('url');
}

sub json {
    my @entries = @_;

    return JSON::Syck::Dump([
        map {
            +{ title => $_->title, thumbnail => thumbnail($_), link => $_->link->href }
        } @entries
    ]);
}
