#!/usr/local/bin/perl
use strict;
use warnings;
use encoding 'utf-8';

use CGI qw/:standard/;
use HTTP::Response::Encoding;
use LWP::UserAgent;
use Params::Validate qw/:all/;
use URI;
use XML::Atom::Feed;
use XML::Atom::Util qw/nodelist/;
use JSON::Syck;
use Template;

my $media = XML::Atom::Namespace->new(media => 'http://search.yahoo.com/mrss/');
my $q = CGI->new;

if ($q->param('json') and $q->param('q')) {
    print
        $q->header( -type => 'application/json', -charset => 'utf-8' ),
        json( search_video($q->param('q'))->entries );
} else {
    print $q->header( -type => 'text/html', -charset => 'utf-8' );

    my $file = $q->param('q')
        ? 'templates/result.html'
        : 'templates/index.html';

    my $vars = $q->param('q')
        ? { result => search_video( $q->param('q') ), thumbnail => \&thumbnail, q => $q->param('q') }
        : {};

    my $template = Template->new;
    $template->process($file, $vars, \my $out) or die $template->error;

    print $out;
}

sub anchor {
    my ($entry) = validate_pos(@_, { isa => 'XML::Atom::Entry'});
    return a( {href => $entry->link->href}, img { src => thumbnail($entry), title => $entry->title } )
}

sub search_video {
    my ($query) = validate_pos(@_, { type => SCALAR });
    my $uri = URI->new('http://gdata.youtube.com/feeds/api/videos');
    $uri->query_form(vq => $query, orderby => 'viewCount' );

    my $ua = LWP::UserAgent->new;
    $ua->agent('sample application/0.01');

    my $res = $ua->get($uri);
    if ($res->is_error) {
        die $res->status_line;
    }

    return _parse_videos( \$res->decoded_content );
}

sub _parse_videos {
    my ($ref_xml) = validate_pos(@_, { type => SCALARREF });
    return XML::Atom::Feed->new($ref_xml) or die XML::Atom->errstr;
}

sub thumbnail {
    my ($entry) = validate_pos(@_, { isa => 'XML::Atom::Entry' });

    my @nodes = nodelist($entry->elem, $media->{uri}, 'thumbnail');
    return $nodes[0]->getAttribute('url');
}

sub json {
    my @entries = @_;

    return JSON::Syck::Dump([
        map {
            +{ title => $_->title, thumbnail => thumbnail($_), link => $_->link->href }
        } @entries
    ]);
}
